<?php

/**
 * Implementation of hook_views_plugins()
 */
function nice_dash_views_plugins() {
  return array(
    'display' => array(
      'nice_dash_dashboard_widget' => array(
        'title' => t('Dashboard widget'),
        'help' => t('Creates a dashboard display.'),
        'handler' => 'nice_dash_views_plugin_display_dashboard_widget',
        'theme' => 'views_view',
        'theme path' => drupal_get_path('module', 'views') . '/theme',
        'nice_dash_dashboard_widget' => TRUE,
        'use ajax' => TRUE,
        'use pager' => TRUE,
        'use more' => FALSE,
        'accept attachments' => TRUE,
        'admin' => t('Dashboard widget'),
        'help topic' => 'display-default',
        'path' => drupal_get_path('module', 'nice_dash') . '/includes',
      ),
    ),
  );
}

/**
 * Implementation of hook_views_default_views().
 */
function nice_dash_views_default_views() {
  module_load_include('inc', 'nice_dash', 'nice_dash.registry');
  $path = drupal_get_path('module', 'nice_dash');
  $registry = _nice_dash_views_registry();
  $views = array();

  foreach ($registry as $key => $value) {
    require_once($path .'/'. $value['file']);

    $function = 'nice_dash_'. $key .'_views';
    if (function_exists($function)) {
      $view = $function();
      $views = array_merge($view, $views);
    }
  }

  return $views;
}

function nice_dash_views_default_views_alter(&$views) {
  if (isset($views['nice_dash_content_manager'])) {
    if (module_exists('translation')) {
      $views['nice_dash_content_manager']->display['default']->display_options['fields']['language'] = array(
        'label' => 'Language',
        'alter' => array(
          'alter_text' => 0,
          'text' => '',
          'make_link' => 0,
          'path' => '',
          'link_class' => '',
          'alt' => '',
          'prefix' => '',
          'suffix' => '',
          'target' => '',
          'help' => '',
          'trim' => 0,
          'max_length' => '',
          'word_boundary' => 1,
          'ellipsis' => 1,
          'strip_tags' => 0,
          'html' => 0,
        ),
        'empty' => '',
        'hide_empty' => 0,
        'empty_zero' => 0,
        'link_to_node' => 0,
        'exclude' => 0,
        'id' => 'language',
        'table' => 'node',
        'field' => 'language',
        'override' => array(
          'button' => 'Overwrite',
        ),
        'relationship' => 'none',
      );

      $no_language = array('***NO_LANGUAGE***' => '***NO_LANGUAGE***');
      $language_list = $no_language + locale_language_list('language');
      $views['nice_dash_content_manager']->display['default']->display_options['filters']['language'] = array(
        'operator' => 'in',
        'value' => $language_list,
        'group' => '0',
        'exposed' => TRUE,
        'expose' => array(
          'use_operator' => 0,
          'operator' => 'language_op',
          'identifier' => 'language',
          'label' => 'Language',
          'optional' => 1,
          'single' => 1,
          'remember' => 1,
          'reduce' => 1,
        ),
        'id' => 'language',
        'table' => 'node',
        'field' => 'language',
        'relationship' => 'none',
      );

      $views['nice_dash_content_manager']->display['default']->display_options['style_options']['columns']['language'] = 'language';
      $views['nice_dash_content_manager']->display['default']->display_options['style_options']['info']['language'] = array(
        'sortable' => 1,
        'separator' => '',
      );
    }
  }
}

/**
 * Return views registry.
 */
function _nice_dash_views_registry() {
  $views = array();

  $views['content'] = array(
    'file' => 'views/nice_dash_content.views.inc',
  );
  $views['content_manager'] = array(
    'file' => 'views/nice_dash_content_manager.views.inc',
  );
  $views['user'] = array(
    'file' => 'views/nice_dash_user.views.inc',
  );

  if (module_exists('scheduler')) {
    $views['scheduler'] = array(
      'file' => 'views/nice_dash_scheduler.views.inc',
    );
  }
  if (module_exists('comment')) {
    $views['comment'] = array(
      'file' => 'views/nice_dash_comment.views.inc',
    );
  }
  return $views;
}