<?php

/**
 * @file
 * Registry file for Nice dashboard.
 * Contains functions which are only called after a cache clear.
 */


/**
 * Fetch dashboards.
 */
function _nice_dash_get_dashboards() {
  $rows = array();
  $result = db_query("SELECT d.did, d.name, d.machine_name, d.weight, d.access, d.status
    FROM {nice_dash_dashboards} d
    ORDER BY d.weight ASC");

  while ($row = db_fetch_object($result)) {
    $rows[$row->did] = $row;
  }

  return $rows;
}

/**
 * Fetch all widgets.
 */
function _nice_dash_get_all_widgets() {

  // Fetch code widgets.
  foreach (nice_dash_get_plugin() as $plugin) {

    $object = nice_dash_get_plugin($plugin['name']);
    if (!$object->is_enabled() || !method_exists($object, 'widgets')) {
      continue;
    }

    $widgets = $object->widgets();
    foreach ($widgets as $key => $value) {

      $wid = _nice_dash_widget_wid($key);

      if (!$wid) {
        $widget = new stdClass();
        $widget->custom = 0;
        $widget->widget_key = $key;
        $widget->widget_plugin = $plugin['name'];
        $widget->description = '';

        if (is_array($value)) {
          $widget->title = $value['title'];
          $widget->data = $value['data'];
        }
        else {
          $widget->title = $value;
        }
        drupal_write_record('nice_dash_widgets', $widget);
      }
    }
  }

  // Fetch db widgets.
  $widgets = array();
  $result = db_query("SELECT * FROM {nice_dash_widgets}");
  while ($row = db_fetch_object($result)) {
    $widgets[$row->wid] = $row;
  }

  $plugin_visibilty = unserialize(variable_get('nice_dash_visibility', ''));
  foreach($widgets as $key => $value) {
    if ($plugin_visibilty[$value->widget_plugin] == '0') {
      unset($widgets[$key]);
    }
  }

  return $widgets;
}

/**
 * Helper function to check if a widget key exists in the db.
 */
function _nice_dash_widget_wid($key) {
  return db_result(db_query("SELECT wid FROM {nice_dash_widgets} WHERE widget_key = '%s'", $key));
}

/**
 * Fetch all custom widgets.
 */
function _nice_dash_get_custom_widgets() {
  $widgets = array();

  $result = db_query("SELECT * FROM {nice_dash_widgets} WHERE custom = 1");
  while ($row = db_fetch_object($result)) {
    $widgets[$row->wid] = $row;
  }

  return $widgets;
}

/**
 * Return menu items.
 */
function _nice_dash_menu() {

  //admin
  $items['admin/build/dashboard'] = array(
    'title' => 'Dashboard',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_dashboard_overview_page'),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_NORMAL_ITEM,
    'file' => 'nice_dash.pages.inc'
  );

  //admin dashboards
  $items['admin/build/dashboard/dashboards'] = array(
    'title' => 'Dashboards',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_dashboard_overview_page'),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => 0,
    'file' => 'nice_dash.pages.inc'
  );

  $items['admin/build/dashboard/dashboards/list'] = array(
    'title' => 'Overview',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => 0
  );

  $items['admin/build/dashboard/dashboards/add'] = array(
    'title' => 'Create new dashboard',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_dashboard_settings_form'),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_LOCAL_TASK,
    'weight' => 1,
    'file' => 'nice_dash.pages.inc'
  );

  $items['admin/build/dashboard/dashboards/%nice_dash_dashboard/edit'] = array(
    'title' => 'Edit dashboard',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_dashboard_settings_form', 4),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_CALLBACK,
    'file' => 'nice_dash.pages.inc'
  );

  $items['admin/build/dashboard/dashboards/%nice_dash_dashboard/delete'] = array(
    'title' => 'Delete dashboard',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_dashboard_delete_form', 4),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_CALLBACK,
    'file' => 'nice_dash.pages.inc'
  );

  //admin widgets
  $items['admin/build/dashboard/widgets'] = array(
    'title' => 'Widgets',
    'page callback' => 'nice_dash_widget_overview_page',
    'access arguments' => array('administer block widgets'),
    'type' => MENU_LOCAL_TASK,
    'weight' => 1,
    'file' => 'nice_dash.pages.inc'
  );

  $items['admin/build/dashboard/widgets/list'] = array(
    'title' => 'Overview',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => 0
  );

  $items['admin/build/dashboard/widgets/add'] = array(
    'title' => 'Create widget',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_widget_settings_form'),
    'access arguments' => array('administer block widgets'),
    'type' => MENU_LOCAL_TASK,
    'weight' => 2,
    'file' => 'nice_dash.pages.inc'
  );

  //TODO: menu loader
  $items['admin/build/dashboard/widgets/%/delete'] = array(
    'title' => 'Delete widget',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_widget_delete_form', 4),
    'access arguments' => array('administer block widgets'),
    'type' => MENU_CALLBACK,
    'file' => 'nice_dash.pages.inc'
  );

  $items['admin/build/dashboard/widgets/%/edit'] = array(
    'title' => 'Edit widget',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_widget_settings_form', 4),
    'access arguments' => array('administer block widgets'),
    'type' => MENU_CALLBACK,
    'file' => 'nice_dash.pages.inc'
  );

  //admin settings
  $items['admin/build/dashboard/settings'] = array(
    'title' => 'Settings',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_settings'),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_LOCAL_TASK,
    'weight' => 2,
    'file' => 'nice_dash.pages.inc'
  );

  //admin plugins
  $items['admin/build/dashboard/plugins'] = array(
    'title' => 'Plugins',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('nice_dash_plugins_settings_form'),
    'access arguments' => array('administer nice dashboard'),
    'type' => MENU_LOCAL_TASK,
    'weight' => 3,
    'file' => 'nice_dash.pages.inc'
  );

  //dashboards
  $items['admin/dashboard'] = array(
    'title' => 'Dashboard',
    'page callback' => '_nice_dash_dashboard_page',
    'access callback' => '_nice_dash_dashboard_access',
    'type' => MENU_NORMAL_ITEM,
    'file' => 'nice_dash.pages.inc',
    'weight' => -100,
  );

  // Implement all the dashboard menu calls.
  $dashboards = nice_dash_get_object();

  if (count($dashboards) > 0) {
    foreach ($dashboards as $dashboard) {
      $items['admin/dashboard/'. $dashboard->machine_name] = array(
        'title' => $dashboard->name,
        'page callback' => 'nice_dash_dashboard_page',
        'page arguments' => array(2),
        'access callback' => 'nice_dash_dashboard_access',
        'access arguments' => array($dashboard),
        'type' => MENU_LOCAL_TASK,
        'weight' => $dashboard->weight,
        'file' => 'nice_dash.pages.inc'
      );
    }
  }
  
  // Support for the google analytics plugin
  $items['admin/dashboard/ga'] = array(
    'page callback' => 'nice_dash_ga_json',
    'access arguments' => array('view nice dashboard'),
    'type' => MENU_CALLBACK,
    'file path' => drupal_get_path('module','nice_dash').'/plugins/nice_dash_ga/',
    'file' => 'nice_dash_ga.pages.inc'
  );
 
  return $items;
}

/**
 * Alter menu items.
 */
function _nice_dash_menu_alter(&$items) {
  if (variable_get('nice_dash_alter_admin_path', FALSE)) {
    $items['admin'] = array(
      'title' => 'Dashboard',
      'page callback' => '_nice_dash_dashboard_page',
      'access callback' => '_nice_dash_dashboard_access',
      'type' => MENU_NORMAL_ITEM,
      'file' => 'nice_dash.pages.inc',
      'file path' => drupal_get_path('module', 'nice_dash'),
    );
  }
}

/**
 * Return theming functions.
 */
function _nice_dash_theme() {
  $path = drupal_get_path('module', 'nice_dash');

  return array(
    'nice_dash_component' => array(
      'arguments' => array('title' => NULL, 'description' => NULL, 'content' => NULL),
      'template' => 'nice-dash-dashboard-component',
      'file' => 'theme.inc',
      'path' => $path .'/theme',
    ),
    'nice_dash_page' => array(
      'arguments' => array('regions' => NULL),
      'template' => 'nice-dash-dashboard',
      'file' => 'theme.inc',
      'path' => $path .'/theme',
    ),
    'nice_dash_settings_form' => array(
      'arguments' => array('form' => NULL),
      'template' => 'nice-dash-dashboard-settings-form',
      'file' => 'theme.inc',
      'path' => $path .'/theme',
    ),
    'nice_dash_overview' => array(
      'arguments' => array('form' => NULL),
      'template' => 'nice-dash-dashboard-overview',
      'file' => 'theme.inc',
      'path' => $path .'/theme',
    ),
  );
}

/**
 * Return plugins.
 */
function _nice_dash_nice_dash_plugins() {
  $plugins = array();

  $plugins['nice_dash'] = array(
    'handler' => array(
      'path' => drupal_get_path('module', 'nice_dash') .'/plugins/nice_dash',
      'file' => 'nice_dash.inc',
      'class' => 'nice_dash',
    ),
  );
  $plugins['nice_dash_content'] = array(
    'handler' => array(
      'path' => drupal_get_path('module', 'nice_dash') .'/plugins/nice_dash_content',
      'file' => 'nice_dash_content.inc',
      'class' => 'nice_dash_content',
    ),
  );
  $plugins['nice_dash_ga'] = array(
    'handler' => array(
      'path' => drupal_get_path('module', 'nice_dash') .'/plugins/nice_dash_ga',
      'file' => 'nice_dash_ga.inc',
      'class' => 'nice_dash_ga',
    ),
  );
  $plugins['nice_dash_views'] = array(
    'handler' => array(
      'path' => drupal_get_path('module', 'nice_dash') .'/plugins/nice_dash_views',
      'file' => 'nice_dash_views.inc',
      'class' => 'nice_dash_views',
    ),
  );
  return $plugins;
}
