<?php

/**
 * @file
 * Google analytics plugin for Nice dashboard.
 */
class nice_dash_ga {

  /**
   * Enabled method.
   */
  function is_enabled() {
    return TRUE;
  }

  function form_alter(&$form, &$form_state) {
    $form['google_analytics'] = array(
        '#type' => 'fieldset',
        '#title' => t('Google analytics settings'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
    );

    // If authenticated show form
    $token = variable_get('nice_dash_ga_token', '');

    if (!empty($token)) {
      module_load_include('inc', 'nice_dash', 'plugins/nice_dash_ga/classes/gapi.class');
      $ga = new gapi(NULL, NULL, $token);
      $ga->requestAccountData();

      $options = array();
      $options[] = t('No account');
      foreach ($ga->getResults() as $result) {
        $options[$result->getProfileId()] = check_plain($result);
      }

      if (!count($options)) {
        $form['google_analytics']['notice'] = array(
            '#value' => t("Currently you don't have any google analytics accounts")
        );
      } else {
        $form['google_analytics']['nice_dash_ga_account'] = array(
            '#type' => 'select',
            '#title' => t('Site account'),
            '#default_value' => variable_get('nice_dash_ga_account', ''),
            '#options' => $options,
            '#description' => t('Choose which account will be used to aggregate statistics.'),
        );
      }
    } else {
      // Capture token
      if (isset($_GET['token'])) {
        $headers = array('Authorization' => 'AuthSub token="' . $_GET['token'] . '"');
        $url = 'https://www.google.com/accounts/AuthSubSessionToken';

        $response = drupal_http_request($url, $headers);
        list($temp, $token) = explode('=', $response->data, 2);

        if (empty($token)) {
          watchdog('nice_dash', t('Failed to receive AuthSub session token.'));
          drupal_set_message(t('Failed to receive AuthSub session token.'), 'error');
        } else {
          variable_set('nice_dash_ga_token', $token);
          return drupal_goto('admin/build/dashboard/settings');
        }
      } else {
        // Build google auth link
        $next_url = url('admin/build/dashboard/settings', array('absolute' => TRUE));
        $url = 'https://www.google.com/accounts/AuthSubRequest?session=1';
        $url .= '&next=';
        $url .= urlencode($next_url);
        $url .= "&scope=";
        $url .= urlencode('https://www.google.com/analytics/feeds');

        $output = '';
        $output .= '<p>'. t('You have to authenticate with Google first. Before you can use the widgets.') .'</p>';
        $output .= l(t('Begin authentication'), $url);

        $form['google_analytics']['google_auth'] = array(
            '#value' => $output,
        );
      }
    }
  }

  /**
   * Widgets method.
   */
  function widgets() {
    return array(
        'ga_overview' => t('Analytics overview'),
        'ga_visits' => t('Visitor statistics'),
        'ga_browsers' => t('Browser statistics'),
        'ga_map' => t('Visitor map'),
        'ga_pages' => t('Page statistics'),
        'ga_entrances' => t('Entrance statistics'),
        'ga_exits' => t('Exit statistics'),
        'ga_sources' => t('Source statistics'),
        'ga_keywords' => t('Keyword statistics'),
        'ga_hours' => t('Hour statistics'),
    );
  }

  function nice_dash_ga_widget_ga_overview() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('visitors', 'pageviews', 'bounces', 'newvisits'));
    $output = '';

    $output .= '<div id="stats-visitors" class="float-left overview"></div>';
    $output .= '<br class="clear-both" />';
    $output .= '<div id="stats-pageviews" class="float-left overview"></div>';
    $output .= '<br class="clear-both" />';
    $output .= '<div id="stats-bounces" class="float-left overview"></div>';
    $output .= '<br class="clear-both" />';
    $output .= '<div id="stats-new-visits" class="float-left overview"></div>';
    $output .= '<br class="clear-both" />';

    return array(
        'title' => t('Analytics overview'),
        'description' => t('Summary of useful statistics.'),
        'content' => '<div id="ga-overview">' . $output . '</div>',
    );
  }

  function nice_dash_ga_widget_ga_browsers() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('browser'));
    $output = '';

    $output .= '<div class="stats-wrapper"><div id="stats-browsers"></div></div>';

    return array(
        'title' => t('Browser statistics'),
        'description' => t('Top 10 of browsers visitors are using to view this website.'),
        'content' => $output,
    );
  }

  /**
   * Google analytics.
   */
  function nice_dash_ga_widget_ga_visits() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('visits'));
    $output = '';

    $output .= '<div class="stats-wrapper"><div id="stats-visits"></div></div>';

    return array(
        'title' => t('Visitor statistics'),
        'description' => t('Overview of visits and pageviews from the last month'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_map() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('geo'));
    $output = '';

    $output .= '<div id="stats-geo"></div>';

    return array(
        'title' => t('Geographical statistics'),
        'description' => t('Geographical overview of site visitors'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_pages() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('pages'));
    $output = '';

    $output .= '<div id="stats-pages"></div>';

    return array(
        'title' => t('Page statistics'),
        'description' => t('Overview of per page statistics.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_entrances() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('entrances'));
    $output = '';

    $output .= '<div id="stats-entrances"></div>';

    return array(
        'title' => t('Entrance statistics'),
        'description' => t('Overview of pages on which visitors enter your site.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_exits() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('exits'));
    $output = '';

    $output .= '<div id="stats-exits"></div>';

    return array(
        'title' => t('Exit statistics'),
        'description' => t('Overview of pages on which visitors exit your site.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_sources() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('sources'));
    $output = '';

    $output .= '<div id="stats-sources"></div>';

    return array(
        'title' => t('Source statistics'),
        'description' => t('Overview of which sites direct traffic to your site.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_keywords() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('keywords'));
    $output = '';

    $output .= '<div id="stats-keywords"></div>';

    return array(
        'title' => t('Keyword statistics'),
        'description' => t('Overview of what keywords are used to find your website.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_widget_ga_hours() {
    $this->nice_dash_ga_add_assets();
    $this->nice_dash_ga_register(array('hours'));
    $output = '';

    $output .= '<div class="stats-wrapper"><div id="stats-hours"></div></div>';

    return array(
        'title' => t('Hour statistics'),
        'description' => t('Overview of site activity per hour.'),
        'content' => $output,
    );
  }

  function nice_dash_ga_add_assets() {
    static $assets_added;

    if ($assets_added) {
      return;
    } else {
      drupal_add_js(drupal_get_path('module', 'nice_dash') . '/plugins/nice_dash_ga/js/nice_dash_ga.js');
      drupal_add_css(drupal_get_path('module', 'nice_dash') . '/plugins/nice_dash_ga/css/nice_dash_ga.css');
      $assets_added = TRUE;
    }
  }
  
  function nice_dash_ga_register($widgets) {
    foreach((array) $widgets as $widget) {
      drupal_add_js(array('nice_dash' => array('ga_widgets' => $widget)), 'setting');
    }
  }

}