<?php

/**
 * Ajax callback for GA requests
 */
function nice_dash_ga_json() {
   module_load_include('inc', 'nice_dash', 'plugins/nice_dash_ga/classes/gapi.class');

   switch($_GET['type']) {
    case 'visits':
      $data = nice_dash_ga_stats_visits();
      break;
    case 'pageviews':
      $data = nice_dash_ga_stats_pageviews();
      break;
    case 'bounces':
      $data = nice_dash_ga_stats_bounces();
      break;
    case 'new_visits':
      $data = nice_dash_ga_stats_new_visits();
      break;
    case 'browser':
      $data = nice_dash_ga_stats_browsers();
      break;
    case 'geo':
      $data = nice_dash_ga_stats_geo();
      break;
    case 'pages':
      $data = nice_dash_ga_stats_pages();
      break;
    case 'entrances':
      $data = nice_dash_ga_stats_entrances();
      break;
    case 'exits':
      $data = nice_dash_ga_stats_exits();
      break;
    case 'sources':
      $data = nice_dash_ga_stats_sources();
      break;
    case 'mediums':
      $data = nice_dash_ga_stats_mediums();
      break;
    case 'keywords':
      $data = nice_dash_ga_stats_keywords();
      break;
    case 'hours':
      $data = nice_dash_ga_stats_hours();
      break;
    default: $data = FALSE;
  }

  return drupal_json(array('status' => 1, 'data' => $data));
}

function nice_dash_ga_stats_geo() {
  $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('country'), // Dimensions
    array('pageviews', 'visits'), // Metrics
    array('-visits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'country' => $result->getCountry(),
      'visits' => $result->getVisits()
    );
  }

  return $data;
}

function nice_dash_ga_stats_browsers() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));
  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('browser', 'browserversion'), // Dimensions
    array('visits'), // Metrics
    array('-visits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    10 // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'browser' => $result->getBrowser(). ' '.$result->getBrowserVersion(),
      'visits' => $result->getVisits()
    );
  }

  return $data;
}

function nice_dash_ga_stats_visits() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(variable_get('nice_dash_ga_account',''),array('date'),array('visits','pageviews'),array('date'));
  $data = array();

  foreach($ga->getResults() as $result) {
    $date = $result->getDate();
    $date = mktime(0, 0, 0, substr($date, 4, 2), substr($date, 6, 2), substr($date, 0, 4));
    $date = format_date($date, 'custom', 'j M');

    $data['results'][] = array(
      'date' => $date,
      'visits' => $result->getVisits(),
      'pageviews' => $result->getPageViews(),
    );
  }

  $data['totalVisits'] = $ga->getVisits();

  return $data;
}

function nice_dash_ga_stats_pageviews() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(variable_get('nice_dash_ga_account',''),array('date'),array('pageviews'),array('date'));
  $data = array();

  foreach($ga->getResults() as $result) {
    $data['results'][] = array(
      'date' => $result->getDate(),
      'pageviews' => $result->getPageviews()
    );
  }

  $data['totalResults'] = $ga->getTotalResults();
  $data['totalPageViews'] = $ga->getPageviews();
  $data['updatedDate'] = $ga->getUpdated();

  return $data;
}

function nice_dash_ga_stats_bounces() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(variable_get('nice_dash_ga_account',''),array('date'),array('bounces','visits'),array('date'));
  $data = array();

  foreach($ga->getResults() as $result) {
    $data['results'][] = array(
      'day' => $result->getDate(),
      'bounces' => $result->getBounces(),
      'visits' => $result->getVisits(),
      'rate' => ($result->getBounces() / $result->getVisits()) * 100,
    );
  }
  $data['totalRate'] = round(($ga->getBounces() / $ga->getVisits()) * 100, 2) .'%';
  return $data;
}

function nice_dash_ga_stats_new_visits() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(variable_get('nice_dash_ga_account',''),array('date'),array('visits','newVisits'),array('date'));
  $data = array();

  foreach($ga->getResults() as $result) {
    $data['results'][] = array(
      'day' => $result->getDate(),
      'rate' => ($result->getNewVisits() / $result->getVisits()) * 100,
    );
  }
  $data['totalRate'] = round(($ga->getNewVisits() / $ga->getVisits()) * 100, 2) . '%';
  return $data;
}

function nice_dash_ga_stats_pages() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('pagepath'), // Dimensions
    array('pageviews'), // Metrics
    array('-pageviews'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'pagepath' => $result->getPagePath(),
      'pageviews' => $result->getPageViews()
    );
  }

  return $data;
}

function nice_dash_ga_stats_entrances() {
     $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('landingpagepath'), // Dimensions
    array('entrances'), // Metrics
    array('-entrances'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'entrances' => $result->getEntrances(),
      'landingpagepath' => $result->getLandingPagePath()
    );
  }

  return $data;
}

function nice_dash_ga_stats_exits() {
     $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('exitpagepath'), // Dimensions
    array('exits'), // Metrics
    array('-exits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'exits' => $result->getExits(),
      'exitpagepath' => $result->getExitPagePath()
    );
  }

  return $data;
}

function nice_dash_ga_stats_sources() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('source'), // Dimensions
    array('visits'), // Metrics
    array('-visits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'visits' => $result->getVisits(),
      'source' => $result->getSource()
    );
  }

  return $data;
}

function nice_dash_ga_stats_keywords() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('keyword'), // Dimensions
    array('visits'), // Metrics
    array('-visits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'visits' => $result->getVisits(),
      'keyword' => $result->getKeyword()
    );
  }

  return $data;
}

function nice_dash_ga_stats_hours() {
    $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('hour'), // Dimensions
    array('visits'), // Metrics
    array('-hour'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'visits' => $result->getVisits(),
      'hour' => $result->getHour().':00'
    );
  }

  return $data;
}

function nice_dash_ga_stats_mediums() {
  $ga = new gapi(null, null, variable_get('nice_dash_ga_token', ''));

  $ga->requestReportData(
    variable_get('nice_dash_ga_account',''),
    array('medium'), // Dimensions
    array('visits'), // Metrics
    array('-visits'), // Sort
    NULL, // Filter
    NULL, // Start date
    NULL, // End date
    1, // Start index
    NULL // Limit results
  );
  $data = array();

  $labels = array(
    '(none)' => t('Direct traffic'),
    'referral' => t('Referring sites'),
    'organic' => t('Search engines'),
  );

  foreach($ga->getResults() as $result) {
    $data[] = array(
      'medium' => $labels[$result->getMedium()],
      'visits' => $result->getVisits()
    );
  }

  return $data;
}
